"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFetch = exports.getHttpsFetch = exports.getHttpFetch = exports.newHttpsAgent = exports.newHttpAgent = exports.getKeepAliveAgentOptions = exports.isFetchKeepaliveEnabled = void 0;
const agentkeepalive = require("agentkeepalive");
const node_fetch_1 = require("node-fetch");
// whether we want to reuse sockets in the fetch client
// this is expecially useful to avoid SNAT exhaustion in Azure
// see https://blog.botframework.com/2018/03/05/fix-snat-exhaustion-node-js-bots/
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
const isFetchKeepaliveEnabled = (env) => env.FETCH_KEEPALIVE_ENABLED === "true";
exports.isFetchKeepaliveEnabled = isFetchKeepaliveEnabled;
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
const getKeepAliveAgentOptions = (env) => ({
    freeSocketTimeout: env.FETCH_KEEPALIVE_FREE_SOCKET_TIMEOUT === undefined
        ? undefined
        : parseInt(env.FETCH_KEEPALIVE_FREE_SOCKET_TIMEOUT, 10),
    keepAlive: true,
    keepAliveMsecs: env.FETCH_KEEPALIVE_KEEPALIVE_MSECS === undefined
        ? undefined
        : parseInt(env.FETCH_KEEPALIVE_KEEPALIVE_MSECS, 10),
    maxFreeSockets: env.FETCH_KEEPALIVE_MAX_FREE_SOCKETS === undefined
        ? undefined
        : parseInt(env.FETCH_KEEPALIVE_MAX_FREE_SOCKETS, 10),
    maxSockets: env.FETCH_KEEPALIVE_MAX_SOCKETS === undefined
        ? undefined
        : parseInt(env.FETCH_KEEPALIVE_MAX_SOCKETS, 10),
    socketActiveTTL: env.FETCH_KEEPALIVE_SOCKET_ACTIVE_TTL === undefined
        ? undefined
        : parseInt(env.FETCH_KEEPALIVE_SOCKET_ACTIVE_TTL, 10),
    timeout: env.FETCH_KEEPALIVE_TIMEOUT === undefined
        ? undefined
        : parseInt(env.FETCH_KEEPALIVE_TIMEOUT, 10),
});
exports.getKeepAliveAgentOptions = getKeepAliveAgentOptions;
// We need the following two exports to prevent the caller module
// to access the unptached version of agentkeepalive which has a bug
// @see https://github.com/node-modules/agentkeepalive/issues/76
// HTTP-only agent, with optional keepalive agent
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
const newHttpAgent = (httpOptions) => new agentkeepalive(httpOptions);
exports.newHttpAgent = newHttpAgent;
// HTTPs-only agent, with optional keepalive agent
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
const newHttpsAgent = (httpsOptions) => new agentkeepalive.HttpsAgent(httpsOptions);
exports.newHttpsAgent = newHttpsAgent;
// Returns a fetch instance backed by a keepalive-enabled HTTP agent
const getKeepaliveHttpFetch = (httpOptions) => {
    // custom HTTP agent that will reuse sockets
    // see https://github.com/node-modules/agentkeepalive#new-agentoptions
    const httpAgent = (0, exports.newHttpAgent)(httpOptions);
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    return (input, init) => {
        const initWithKeepalive = Object.assign(Object.assign({}, (init === undefined ? {} : init)), { agent: httpAgent });
        // need to cast to any since node-fetch has a slightly different type
        // signature that DOM's fetch
        // TODO: possibly avoid using DOM's fetch type altoghether?
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return (0, node_fetch_1.default)(input, initWithKeepalive);
    };
};
// Returns a fetch instance backed by a keepalive-enabled HTTP agent
const getKeepaliveHttpsFetch = (httpsOptions) => {
    // custom HTTP agent that will reuse sockets
    // see https://github.com/node-modules/agentkeepalive#new-agentoptions
    const httpAgent = (0, exports.newHttpsAgent)(httpsOptions);
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type, sonarjs/no-identical-functions
    return (input, init) => {
        const initWithKeepalive = Object.assign(Object.assign({}, (init === undefined ? {} : init)), { agent: httpAgent });
        // need to cast to any since node-fetch has a slightly different type
        // signature that DOM's fetch
        // TODO: possibly avoid using DOM's fetch type altoghether?
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        return (0, node_fetch_1.default)(input, initWithKeepalive);
    };
};
// HTTP-only fetch, with optional keepalive agent
// Note: extra options are valid only when FETCH_KEEPALIVE_ENABLED=true
const getHttpFetch = (env, extraOptions = {}) => (0, exports.isFetchKeepaliveEnabled)(env)
    ? getKeepaliveHttpFetch(Object.assign(Object.assign({}, (0, exports.getKeepAliveAgentOptions)(env)), extraOptions))
    : // eslint-disable-next-line @typescript-eslint/no-explicit-any
        node_fetch_1.default;
exports.getHttpFetch = getHttpFetch;
// HTTPs-only fetch, with optional keepalive agent
// Note: extra options are valid only when FETCH_KEEPALIVE_ENABLED=true
const getHttpsFetch = (env, extraOptions = {}) => (0, exports.isFetchKeepaliveEnabled)(env)
    ? getKeepaliveHttpsFetch(Object.assign(Object.assign({}, (0, exports.getKeepAliveAgentOptions)(env)), extraOptions))
    : // eslint-disable-next-line @typescript-eslint/no-explicit-any
        node_fetch_1.default;
exports.getHttpsFetch = getHttpsFetch;
// Wraps both HTTP and HTTPS agents and serve the correct one based on the protocol of the request url
// Note: extra options are valid only when FETCH_KEEPALIVE_ENABLED=true
const getFetch = (env, extraOptions = {}) => {
    const httpAgent = (0, exports.getHttpFetch)(env, extraOptions);
    const httpsAgent = (0, exports.getHttpsFetch)(env, extraOptions);
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    return (input, init) => {
        const url = input instanceof URL
            ? input.toString()
            : input instanceof Request
                ? input.url
                : input;
        return url.startsWith("https://")
            ? httpsAgent(input, init)
            : httpAgent(input, init);
    };
};
exports.getFetch = getFetch;
//# sourceMappingURL=agent.js.map