"use strict";
/**
 * Provides an implementation of backoff algorithms to be used when waiting
 * between retries of a failing task.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.calculateExponentialBackoffInterval = exports.DEFAULT_EXPONENTIAL_BACKOFF_MAX = exports.DEFAULT_EXPONENTIAL_BACKOFF_MAX_JITTER = exports.DEFAULT_EXPONENTIAL_BACKOFF_MULTIPLIER = exports.DEFAULT_EXPONENTIAL_BACKOFF_BASE_INTERVAL = void 0;
// for an explanation of these constants, see:
// https://cloud.google.com/storage/docs/exponential-backoff
exports.DEFAULT_EXPONENTIAL_BACKOFF_BASE_INTERVAL = 500;
exports.DEFAULT_EXPONENTIAL_BACKOFF_MULTIPLIER = 2;
exports.DEFAULT_EXPONENTIAL_BACKOFF_MAX_JITTER = 1000;
exports.DEFAULT_EXPONENTIAL_BACKOFF_MAX = (32 * 1000);
/**
 * Returns a function that calculates the backoff interval for each try after
 * a failure.
 *
 * For an explanation of the algorithm, see:
 * https://cloud.google.com/storage/docs/exponential-backoff
 *
 * @param n Is the try count, starts from 0
 */
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function calculateExponentialBackoffInterval(baseInterval = exports.DEFAULT_EXPONENTIAL_BACKOFF_BASE_INTERVAL, multiplier = exports.DEFAULT_EXPONENTIAL_BACKOFF_MULTIPLIER, maxJitter = exports.DEFAULT_EXPONENTIAL_BACKOFF_MAX_JITTER, maxBackoff = exports.DEFAULT_EXPONENTIAL_BACKOFF_MAX) {
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    return (count) => {
        // The jitter helps to avoid cases where many clients get synchronized by
        // some situation and all retry at once, sending requests in synchronized
        // waves.
        const jitter = Math.random() * maxJitter;
        // The wait time is:
        // eslint-disable-next-line extra-rules/no-commented-out-code
        // min((b*(m^n)+random_number_milliseconds), maximum_backoff)
        // with n incremented by 1 for each iteration (request).
        const delayInterval = baseInterval * Math.pow(multiplier, count) + jitter;
        return Math.min(delayInterval, maxBackoff);
    };
}
exports.calculateExponentialBackoffInterval = calculateExponentialBackoffInterval;
//# sourceMappingURL=backoff.js.map