"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DateFromTimestamp = exports.UTCISODateFromString = exports.IsoDateFromString = exports.TimezoneOnlyIsoDateFromString = exports.UtcOnlyIsoDateFromString = exports.patternDateFromString = exports.DateFromString = void 0;
const t = require("io-ts");
const E = require("fp-ts/lib/Either");
const function_1 = require("fp-ts/lib/function");
const strings_1 = require("./strings");
const isDate = (v) => v instanceof Date;
/**
 * Accepts short formats (ie. "2018-10-13")
 * with or without time and timezone parts.
 */
exports.DateFromString = new t.Type("DateFromString", isDate, (v, c) => isDate(v)
    ? t.success(v)
    : (0, function_1.pipe)(t.string.validate(v, c), E.chain((s) => {
        const d = new Date(s);
        return isNaN(d.getTime()) ? t.failure(s, c) : t.success(d);
    })), (a) => a.toISOString());
const patternDateFromString = (pattern, tagName) => new t.Type(tagName, isDate, (v, c) => isDate(v)
    ? t.success(v)
    : (0, function_1.pipe)((0, strings_1.PatternString)(pattern).validate(v, c), E.chain((s) => {
        const d = new Date(s);
        return isNaN(d.getTime()) ? t.failure(s, c) : t.success(d);
    })), (a) => a.toISOString());
exports.patternDateFromString = patternDateFromString;
/**
 * ISO8601 format for dates.
 *
 */
/**
 * Date and time is separated with a capital T.
 * UTC time is defined with a capital letter Z.
 *
 */
const UTC_ISO8601_FULL_REGEX = "^\\d{4}-\\d\\d-\\d\\dT\\d\\d:\\d\\d:\\d\\d(\\.\\d+)?(Z)?$";
/**
 * Date and time is separated with a capital T.
 * Timezone is defined with +/-00:00 format.
 *
 */
const TIMEZONE_ISO8601_FULL_REGEX = "^\\d{4}-\\d\\d-\\d\\dT\\d\\d:\\d\\d:\\d\\d(\\.\\d+)?[+-](\\d{2})\\:(\\d{2})$";
/**
 * Accepts only full ISO8601 format with UTC (Z) timezone
 *
 * ie. "2018-10-13T00:00:00.000Z"
 */
exports.UtcOnlyIsoDateFromString = (0, exports.patternDateFromString)(UTC_ISO8601_FULL_REGEX, "UtcOnlyIsoDateFromString");
/**
 * Accepts only full ISO8601 format with +/-00:00 timezone
 *
 * ie. "2018-10-13T00:00:00.000+00:00"
 */
exports.TimezoneOnlyIsoDateFromString = (0, exports.patternDateFromString)(TIMEZONE_ISO8601_FULL_REGEX, "TimezoneOnlyIsoDateFromString");
exports.IsoDateFromString = t.union([exports.UtcOnlyIsoDateFromString, exports.TimezoneOnlyIsoDateFromString], "IsoDateFromString");
/**
 * @deprecated use IsoDateFromString instead.
 */
exports.UTCISODateFromString = exports.IsoDateFromString;
/**
 * Accepts only a valid timestamp format (number)
 *
 * ie. 1577836800000
 */
exports.DateFromTimestamp = new t.Type("DateFromTimestamp", isDate, (v, c) => isDate(v)
    ? t.success(v)
    : (0, function_1.pipe)(t.number.validate(v, c), E.chain((s) => {
        const d = new Date(s);
        return isNaN(d.getTime()) ? t.failure(s, c) : t.success(d);
    })), (a) => a.valueOf());
//# sourceMappingURL=dates.js.map