"use strict";
/**
 * Helpers for dealing with fetch requests
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.retriableFetch = exports.setFetchTimeout = exports.toFetch = exports.AbortableFetch = void 0;
const TaskEither_1 = require("fp-ts/lib/TaskEither");
const E = require("fp-ts/lib/Either");
const function_1 = require("fp-ts/lib/function");
const promises_1 = require("./promises");
const tasks_1 = require("./tasks");
const tuples_1 = require("./tuples");
// eslint-disable-next-line import/no-internal-modules
require("abort-controller/polyfill");
/**
 * Makes fetch abortable and returns an AbortableFetch.
 *
 * The fetch request can be aborted by calling `AbortController`'s `abort()`.
 *
 * See https://developer.mozilla.org/en-US/docs/Web/API/AbortController/abort
 */
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function AbortableFetch(origFetch = fetch) {
    // a fetch-like function that for each request, returns a tuple with the
    // response and the instance of the abortController associated to the request
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    return (input, init) => {
        // instantiate an abort controller
        const abortController = new AbortController();
        const signal = abortController.signal;
        // augment the provided fetch with the signal controller
        const newInit = Object.assign(Object.assign({}, init), { signal });
        // execute the request
        const response = origFetch(input, newInit);
        return (0, tuples_1.Tuple2)(response, abortController);
    };
}
exports.AbortableFetch = AbortableFetch;
/**
 * Converts an AbortableFetch back to a simple fetch
 */
const toFetch = 
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
(f) => (input, init) => f(input, init).e1;
exports.toFetch = toFetch;
/**
 * Sets a timeout on an AbortableFetch.
 *
 * When the timeout expires, the controller's `abort()` method gets called.
 */
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function setFetchTimeout(timeout, abortableFetch) {
    // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
    return (input, init) => {
        const result = abortableFetch(input, init);
        // abort the fetch request on timeout
        (0, promises_1.timeoutPromise)(timeout)
            .then(() => {
            result.e2.abort();
        })
            .catch((_) => void 0);
        return result;
    };
}
exports.setFetchTimeout = setFetchTimeout;
/**
 * Given a function that wraps a RetriableTask with retries, it returns an
 * fetch that wraps the original fetch with retries on transient errors such
 * as network errors and timeouts.
 */
const retriableFetch = (retryWrapper, shouldAbort = Promise.resolve(false)
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
) => (f) => (input, init) => {
    // wraps the fetch call with a TaskEither type, mapping certain promise
    // rejections to TransientError(s)
    const fetchTask = (0, TaskEither_1.tryCatch)(() => f(input, init), (reason) => {
        // map rejection reason to a transient or permanent error
        if (`${reason}`.toLowerCase().indexOf("aborted") >= 0 ||
            reason.name === "Aborted" ||
            reason.name === "Network request failed" ||
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            reason.type === "aborted") {
            // We return a TransientError in case the request got aborted by
            // a timeout or there was a network request.
            // See possible rejections of fetch here:
            // https://github.com/github/fetch/blob/master/fetch.js#L441
            // TODO: make this logic customizable (e.g. via response status)
            return tasks_1.TransientError;
        }
        // in all other case we return the reject reason
        // TODO: provide reason?
        return new Error("Promise has been rejected");
    });
    // wrap the fetch task with the provided retry wrapper
    const fetchWithRetries = retryWrapper(fetchTask, shouldAbort);
    // return a new promise that gets resolved when the task resolves to a Right
    // or gets rejected in all other cases
    return new Promise((resolve, reject) => {
        fetchWithRetries().then((result) => {
            (0, function_1.pipe)(result, E.fold(reject, resolve));
        }, reject);
    });
};
exports.retriableFetch = retriableFetch;
//# sourceMappingURL=fetch.js.map