import * as t from "io-ts";
import { Tagged } from "./types";
export interface INonEmptyStringTag {
    readonly kind: "INonEmptyStringTag";
}
/**
 * A non-empty string
 */
export declare const NonEmptyString: Tagged<INonEmptyStringTag, string, string, unknown>;
export type NonEmptyString = t.TypeOf<typeof NonEmptyString>;
export interface IWithinRangeStringTag<L extends number, H extends number> {
    readonly lower: L;
    readonly higher: H;
    readonly kind: "IWithinRangeStringTag";
}
/**
 * A string guaranteed to have a length within the range [L,H)
 */
export declare const WithinRangeString: <L extends number, H extends number, T extends IWithinRangeStringTag<L, H>>(l: L, h: H) => Tagged<T, string, string, unknown>;
export type WithinRangeString<L extends number, H extends number> = string & IWithinRangeStringTag<L, H>;
export interface IPatternStringTag<P extends string> {
    readonly pattern: P;
    readonly kind: "IPatternStringTag";
}
/**
 * A string that matches a pattern.
 */
export declare const PatternString: <P extends string, T extends IPatternStringTag<P>>(p: P) => Tagged<T, string, string, unknown>;
export type PatternString<P extends string> = string & IPatternStringTag<P>;
export interface IEmailStringTag {
    readonly kind: "IEmailStringTag";
}
/**
 * A string that represents a valid email address.
 */
export declare const EmailString: Tagged<IEmailStringTag, string, string, unknown>;
export type EmailString = t.TypeOf<typeof EmailString>;
export interface IIPStringTag {
    readonly kind: "IIPStringTag";
}
/**
 * A string that represents an IP (v4 or v6).
 */
export declare const IPString: Tagged<IIPStringTag, string, string, unknown>;
export type IPString = t.TypeOf<typeof IPString>;
/**
 * A string that represents a valid CIDR.
 */
export declare const CIDR: Tagged<IPatternStringTag<`(?:^(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])(?:\\.(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])){3}\\/(3[0-2]|[12]?[0-9])$)|(?:^${string}$)`>, string, string, unknown>;
export type CIDR = t.TypeOf<typeof CIDR>;
/**
 * A valid Fiscal Number (for persons)
 */
export declare const FiscalCode: Tagged<IPatternStringTag<"^[A-Z]{6}[0-9LMNPQRSTUV]{2}[ABCDEHLMPRST][0-9LMNPQRSTUV]{2}[A-Z][0-9LMNPQRSTUV]{3}[A-Z]$">, string, string, unknown>;
export type FiscalCode = t.TypeOf<typeof FiscalCode>;
/**
 * A sandbox Fiscal Code (used for development)
 * Note that this pattern doesn't generate valid fiscal codes
 */
export declare const SandboxFiscalCode: Tagged<IPatternStringTag<"^[A-Z]{6}[0-9]{2}A[0-9]{2}Y[0-9]{3}X$">, string, string, unknown>;
export type SandboxFiscalCode = t.TypeOf<typeof SandboxFiscalCode>;
/**
 * A valid Fiscal Number (for organizations)
 */
export declare const OrganizationFiscalCode: Tagged<IPatternStringTag<"^[0-9]{11}$">, string, string, unknown>;
export type OrganizationFiscalCode = t.TypeOf<typeof OrganizationFiscalCode>;
/**
 * The App version using the format {major}.{minor}.{patch}(.{build})
 * The build value is optional.
 */
export declare const Semver: Tagged<IPatternStringTag<"^((0|[1-9]\\d*)\\.){2}(0|[1-9]\\d*)(\\.(0|[1-9]\\d*)){0,1}$">, string, string, unknown>;
export type Semver = t.TypeOf<typeof Semver>;
/**
 * A valid ULID string
 */
export declare const Ulid: Tagged<IPatternStringTag<"^[0-9a-hjkmnp-tv-zA-HJKMNP-TV-Z]{26}$">, string, string, unknown>;
export type Ulid = t.TypeOf<typeof Ulid>;
