"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Ulid = exports.Semver = exports.OrganizationFiscalCode = exports.SandboxFiscalCode = exports.FiscalCode = exports.CIDR = exports.IPString = exports.EmailString = exports.PatternString = exports.WithinRangeString = exports.NonEmptyString = void 0;
const t = require("io-ts");
const validator = require("validator");
const types_1 = require("./types");
/**
 * A non-empty string
 */
exports.NonEmptyString = (0, types_1.tag)()(t.refinement(t.string, (s) => s.length > 0, "non empty string"));
/**
 * A string guaranteed to have a length within the range [L,H)
 */
const WithinRangeString = (l, h) => (0, types_1.tag)()(t.refinement(t.string, (s) => s.length >= l && s.length < h, `string of length >= ${l} and < ${h}`));
exports.WithinRangeString = WithinRangeString;
/**
 * A string that matches a pattern.
 */
const PatternString = (p) => (0, types_1.tag)()(t.refinement(t.string, (s) => s.match(p) !== null, `string that matches the pattern "${p}"`));
exports.PatternString = PatternString;
/**
 * A string that represents a valid email address.
 */
exports.EmailString = (0, types_1.tag)()(t.refinement(t.string, (s) => validator.isEmail(s, {
    allow_display_name: false,
    allow_utf8_local_part: false,
    require_tld: true,
}), "string that represents an email address"));
/**
 * A string that represents an IP (v4 or v6).
 */
exports.IPString = (0, types_1.tag)()(t.refinement(t.string, validator.isIP, "string that represents an IP address"));
const v4 = "(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])(?:\\.(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9][0-9]|[0-9])){3}\\/(3[0-2]|[12]?[0-9])";
const v6seg = "[0-9a-fA-F]{1,4}";
const v6 = `
(
(?:${v6seg}:){7}(?:${v6seg}|:)|                                // 1:2:3:4:5:6:7::  1:2:3:4:5:6:7:8
(?:${v6seg}:){6}(?:${v4}|:${v6seg}|:)|                         // 1:2:3:4:5:6::    1:2:3:4:5:6::8   1:2:3:4:5:6::8  1:2:3:4:5:6::1.2.3.4
(?:${v6seg}:){5}(?::${v4}|(:${v6seg}){1,2}|:)|                 // 1:2:3:4:5::      1:2:3:4:5::7:8   1:2:3:4:5::8    1:2:3:4:5::7:1.2.3.4
(?:${v6seg}:){4}(?:(:${v6seg}){0,1}:${v4}|(:${v6seg}){1,3}|:)| // 1:2:3:4::        1:2:3:4::6:7:8   1:2:3:4::8      1:2:3:4::6:7:1.2.3.4
(?:${v6seg}:){3}(?:(:${v6seg}){0,2}:${v4}|(:${v6seg}){1,4}|:)| // 1:2:3::          1:2:3::5:6:7:8   1:2:3::8        1:2:3::5:6:7:1.2.3.4
(?:${v6seg}:){2}(?:(:${v6seg}){0,3}:${v4}|(:${v6seg}){1,5}|:)| // 1:2::            1:2::4:5:6:7:8   1:2::8          1:2::4:5:6:7:1.2.3.4
(?:${v6seg}:){1}(?:(:${v6seg}){0,4}:${v4}|(:${v6seg}){1,6}|:)| // 1::              1::3:4:5:6:7:8   1::8            1::3:4:5:6:7:1.2.3.4
(?::((?::${v6seg}){0,5}:${v4}|(?::${v6seg}){1,7}|:))           // ::2:3:4:5:6:7:8  ::2:3:4:5:6:7:8  ::8             ::1.2.3.4
)(%[0-9a-zA-Z]{1,})?                                           // %eth0            %1
\\/(12[0-8]|1[01][0-9]|[1-9]?[0-9])
`
    .replace(/\s*\/\/.*$/gm, "")
    .replace(/\n/g, "")
    .trim();
/**
 * A string that represents a valid CIDR.
 */
exports.CIDR = (0, exports.PatternString)(`(?:^${v4}$)|(?:^${v6}$)`);
/**
 * A valid Fiscal Number (for persons)
 */
exports.FiscalCode = (0, exports.PatternString)("^[A-Z]{6}[0-9LMNPQRSTUV]{2}[ABCDEHLMPRST][0-9LMNPQRSTUV]{2}[A-Z][0-9LMNPQRSTUV]{3}[A-Z]$");
/**
 * A sandbox Fiscal Code (used for development)
 * Note that this pattern doesn't generate valid fiscal codes
 */
exports.SandboxFiscalCode = (0, exports.PatternString)("^[A-Z]{6}[0-9]{2}A[0-9]{2}Y[0-9]{3}X$");
/**
 * A valid Fiscal Number (for organizations)
 */
exports.OrganizationFiscalCode = (0, exports.PatternString)("^[0-9]{11}$");
/**
 * The App version using the format {major}.{minor}.{patch}(.{build})
 * The build value is optional.
 */
exports.Semver = (0, exports.PatternString)("^((0|[1-9]\\d*)\\.){2}(0|[1-9]\\d*)(\\.(0|[1-9]\\d*)){0,1}$");
/**
 * A valid ULID string
 */
exports.Ulid = (0, exports.PatternString)("^[0-9a-hjkmnp-tv-zA-HJKMNP-TV-Z]{26}$");
//# sourceMappingURL=strings.js.map