import * as t from "io-ts";
/**
 * Returns a subset of the input objects fields.
 *
 * @param obj the input object
 * @param props an array of keys to preserve
 */
export declare const pick: <T, K extends keyof T>(props: readonly K[], obj: T) => Pick<T, K>;
/**
 * An io-ts Type tagged with T
 */
export type Tagged<T, A, O = A, I = t.mixed> = t.Type<A & T, O & T, I>;
/**
 * Tags an io-ts type with an interface T
 */
export declare const tag: <T>() => <A, O = A, I = unknown>(type: t.Type<A, O, I>) => Tagged<T, A, O, I>;
/**
 * Removes any extra tags from a type that extends a basic type.
 *
 * Basic types: string, number, boolean
 */
export type UntagBasicType<A> = A extends string ? string : A extends number ? number : A extends boolean ? boolean : never;
/**
 * Returns the passed tagged basic value after converting it to its basic type.
 */
export declare const untag: <T>(a: T) => UntagBasicType<T>;
/**
 * Creates an io-ts Type from an enum
 */
export declare const enumType: <E>(e: object, name: string) => t.Type<E, E, unknown>;
/**
 * Creates an io-ts Type from a ReadonlySet
 */
export declare const readonlySetType: <E>(o: t.Type<E, unknown, unknown>, name: string) => t.Type<ReadonlySet<E>, unknown, unknown>;
export interface ReadOnlyNonEmptySet<T> extends ReadonlySet<T> {
}
/**
 * Creates an io-ts Type from a ReadonlyNonEmptySet
 */
export declare const readonlyNonEmptySetType: <E>(o: t.Type<E, unknown, unknown>, name: string) => t.Type<ReadOnlyNonEmptySet<E>, unknown, unknown>;
/**
 * Returns a new type that has only the F fields of type T.
 */
export type LimitedFields<T, F extends keyof T> = {
    [P in F]: T[P];
};
/**
 *  True when the input is an object (and not array).
 */
export declare const isObject: (o: {}) => boolean;
/**
 * Return an object filtering out keys that point to undefined values.
 */
export declare const withoutUndefinedValues: <T extends Object, K extends keyof T>(obj: T) => T;
/**
 *  Return a new type that validates successfully only
 *  when the instance (object) contains no unknown properties.
 *
 *  See https://github.com/gcanti/io-ts/issues/106
 *
 *  @\required  required properties
 *  @optional   optional object properties
 */
export declare const strictInterfaceWithOptionals: <R extends t.Props, O extends t.Props>(required: R, optional: O, name: string) => t.Type<t.TypeOfProps<R> & t.TypeOfPartialProps<O>, t.OutputOfProps<R> & t.OutputOfPartialProps<O>, unknown>;
/**
 * Sets properties default values when calling t.validate() method on models
 * see https://github.com/gcanti/io-ts/issues/8
 */
export declare const withDefault: <T extends t.Any>(type: T, defaultValue: t.TypeOf<T>) => t.Type<t.TypeOf<T>, any, unknown>;
/**
 * From T omit a set of properties K
 */
export type Omit<T, K extends keyof T> = Pick<T, Exclude<keyof T, K>>;
export type ReplaceProp1<T, P1 extends keyof T, A> = {
    [K in P1]: A;
} & Pick<T, Exclude<keyof T, P1>>;
export type ReplaceProp2<T, P1 extends keyof T, P2 extends keyof T[P1], A> = {
    [K in P1]: ReplaceProp1<T[K], P2, A>;
} & Pick<T, Exclude<keyof T, P1>>;
/**
 * Removes null/undefined types from T[P1]
 */
export type RequiredProp1<T, P1 extends keyof T> = ReplaceProp1<T, P1, NonNullable<T[P1]>>;
/**
 * Removes null/undefined types from T[P1][P2]
 */
export type RequiredProp2<T, P1 extends keyof T, P2 extends keyof T[P1]> = ReplaceProp2<T, P1, P2, NonNullable<T[P1][P2]>>;
export declare const requiredProp1: <A, O, I, P extends keyof A>(type: t.Type<A, O, I>, p: P, name?: string) => t.Type<RequiredProp1<A, P>, O, I>;
export declare const replaceProp1: <A, O, I, P extends keyof A, A1 extends A[P], O1, I1>(type: t.Type<A, O, I>, p: P, typeB: t.Type<A1, O1, I1>, name?: string) => t.Type<ReplaceProp1<A, P, A1>, O, I>;
/**
 * Returns the type `A` if `T` is a `Promise<A>`, or else returns `never`
 */
export type PromiseType<T> = T extends Promise<infer A> ? A : never;
/**
 * Extract the type of the first element of an array
 */
export type Head<T extends ReadonlyArray<unknown>> = T[0];
/**
 * Extract the type of the sub-array of the tail of an array
 */
export type Tail<T extends ReadonlyArray<unknown>> = T extends readonly [
    unknown,
    ...infer Rest
] ? Rest : readonly [];
/**
 * Whether an array is empty or has only one element
 */
export type HasTail<T extends ReadonlyArray<unknown>> = T extends readonly [] | readonly [unknown] ? false : true;
